# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-11-27 07:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0016_sapsuccessfactorsenterprisecustomerconfiguration_additional_locales'),
    ]

    operations = [
        migrations.AddField(
            model_name='sapsuccessfactorsglobalconfiguration',
            name='search_student_api_path',
            field=models.CharField(default='learning/odatav4/searchStudent/v1/Students', max_length=255),
            preserve_default=False,
        ),
    ]
